async def gather(hub):
    """
    Authenticate to the {{cookiecutter.service_name}} with it's traditional cli tool's config/environment
    """
    sub_profiles = {}

    # TODO add profiles based on traditional {{cookiecutter.service_name}} cli tool auth

    return sub_profiles
